using System;
//using System.Collections.Generic;
using System.Text;

using System.Drawing;
using System.Runtime.Serialization;
using System.Security.Permissions;

using Syncfusion.Windows.Forms.Diagram;

namespace SankeyTest
{
    [Serializable()]
    public class MyLink : Link
    {
        # region Members Variables

        protected CompassHeading heading1 = CompassHeading.None;
        protected CompassHeading heading2 = CompassHeading.None;

        private const string HEADING_ONE = "H1";
        private const string HEADING_TWO = "H2";
        # endregion

        public delegate void LinkConnectionChangedEventHandler();
        public event LinkConnectionChangedEventHandler ConnectionChanged;

        # region Constructor and Dispose
        // CompassHeadings 1 and 2 specify the tail and head directions for the orthogonal line
        public MyLink(PointF pt1, CompassHeading heading1, PointF pt2, CompassHeading heading2)
        {
            this.heading1 = heading1;
            this.heading2 = heading2;
            PointF[] pts = new PointF[2] { pt1, pt2 };
            IPoints linkPoints = this.CreateLinkShape(Shapes.OrthogonalLine, pts);

            if (linkPoints != null)
            {
                INode childNode = linkPoints as INode;
                if (childNode != null)
                {
                    this.InsertChild(childNode, 0);
                }
            }
         
        }

        public MyLink(PointF pt1, CompassHeading heading1, PointF pt2, CompassHeading heading2, PointF[] pt3)
        {
            this.heading1 = heading1;
            this.heading2 = heading2;


            PointF[] pts = new PointF[2 + pt3.Length];
            int count = 0;
            pts[count++] = pt1;
            foreach (PointF f in pt3)
            {
                pts[count++] = f;
            }
            pts[count++] = pt2;

            IPoints linkPoints = this.CreateLinkShape(Shapes.OrthogonalLine, pts);

            if (linkPoints != null)
            {
                INode childNode = linkPoints as INode;
                if (childNode != null)
                {
                    this.InsertChild(childNode, 0);
                }
            }
            this.EditStyle.AllowMove = false;
            this.EditStyle.AllowSelect = true;
            this.EditStyle.AllowVertexEdit = false;
        }

        // Serialization constructor
        protected MyLink(SerializationInfo info, StreamingContext context)
            : base(info, context)
        {
            this.heading1 = (CompassHeading)info.GetValue(HEADING_ONE, typeof(CompassHeading));
            this.heading2 = (CompassHeading)info.GetValue(HEADING_TWO, typeof(CompassHeading));
        }

        [SecurityPermission(SecurityAction.Demand, SerializationFormatter = false)]
        protected override void GetObjectData(SerializationInfo info, StreamingContext context)
        {
            info.SetType(this.GetType());
            base.GetObjectData(info, context);
            info.AddValue(HEADING_ONE, this.heading1);
            info.AddValue(HEADING_TWO, this.heading2);
        }

        // Creates a new object that is a copy of the current instance.	
        // An override of the Clone method is required for clipboard copy/paste.
        public override object Clone()
        {
            return new MyLink(this);
        }

        public MyLink(MyLink src)
            : base(src)
        {
            this.heading1 = src.heading1;
            this.heading2 = src.heading2;
        }
        # endregion

        # region Methods

        // Override the Link.CreateLinkShape method and create the CustomOrthogonalLine using the provided CompassHeadings
        protected override IPoints CreateLinkShape(Shapes shapeType, PointF[] pts)
        {
            Shape linkShape = null;
            if ((shapeType == Shapes.OrthogonalLine) && (this.heading1 != CompassHeading.None) && (this.heading2 != CompassHeading.None))
            {
                MyLine Line = new MyLine(pts[0], this.heading1, pts[1], this.heading2);
                Line.AutomaticHeadings = false;
                linkShape = Line;
            }

			if ((linkShape != null) && (null != linkShape.GetPoints()))
			{
				linkShape.SetPoints(linkShape.GetPoints());
			}
//            if ((linkShape != null) && (null != linkShape.GetPoints()))
//            {
//                linkShape.SetPoints(linkShape.GetPoints());
//            }
            return linkShape;
        }

        /// <summary>
        /// Called when a vertex is moved.
        /// </summary>
        /// <param name="evtArgs">Event arguments.</param>
        /// <remarks>
        /// <para>
        /// We have used this method to set the page dirty on Move vertex.
        /// </para>
        /// <para>
        /// Forwards the event notification to the parent node's
        /// <see cref="Syncfusion.Windows.Forms.Diagram.IDispatchNodeEvents.MoveVertex"/>
        /// method.
        /// </para>
        /// </remarks>
        /// 

        protected override void OnMoveVertex(VertexMoveEventArgs evtArgs)
        {
            base.OnMoveVertex(evtArgs);
            if (null != ConnectionChanged)
            {
                ConnectionChanged();
            }

        }

        /// <summary>
        /// Create a new connection between the given ports.
        /// </summary>
        /// <param name="sourcePort">First port to connect.</param>
        /// <param name="targetPort">Second port to connect.</param>
        /// <returns>New connection object or NULL on failure.</returns>
        /// <remarks>
        /// <para>
        /// This method first calls the
        /// <see cref="Syncfusion.Windows.Forms.Diagram.Port.AcceptConnection"/>
        /// method on each port in order to ask permission to create the
        /// connection. If both ports consent to the connection, a new
        /// <see cref="Syncfusion.Windows.Forms.Diagram.Connection"/> object
        /// is created to bind the two ports together. Then the connection is
        /// added to the
        /// <see cref="Syncfusion.Windows.Forms.Diagram.SymbolBase.Connections"/>
        /// collection of both port containers.
        /// </para>
        /// <seealso cref="Syncfusion.Windows.Forms.Diagram.SymbolBase.Disconnect"/>
        /// <seealso cref="Syncfusion.Windows.Forms.Diagram.SymbolBase.Connections"/>
        /// <seealso cref="Syncfusion.Windows.Forms.Diagram.Connection"/>
        /// <seealso cref="Syncfusion.Windows.Forms.Diagram.Port"/>
        /// <seealso cref="Syncfusion.Windows.Forms.Diagram.IPortContainer"/>
        /// </remarks>
        public override Connection Connect(Port sourcePort, Port targetPort)
        {
            ///This code snipit recalculate the compass heading for a connection 
            ///whenever a connection is moved.
            if (sourcePort is LinkPort)
            {
                if (((this.IsTailPortConnected) && (!this.IsHeadPortConnected)) ||
                    ((this.IsHeadPortConnected) && (!this.IsTailPortConnected)))
                {
                    MyLinkSupport myLinkSupport = new MyLinkSupport();

                    if (sourcePort == (sourcePort.Parent as MyLink).TailPort)
                    {
                        (sourcePort.Parent as MyLink).heading1 = myLinkSupport.CalculateCompassHeading(targetPort);
                        (this.Points as MyLine).HeadingEndPoint1 = myLinkSupport.CalculateCompassHeading(targetPort);
                    }
                    else if (sourcePort == (sourcePort.Parent as MyLink).HeadPort)
                    {
                        (sourcePort.Parent as MyLink).heading2 = myLinkSupport.CalculateCompassHeading(targetPort);
                        (this.Points as MyLine).HeadingEndPoint2 = myLinkSupport.CalculateCompassHeading(targetPort);
                    }
                    (this.Points as MyLine).SegmentMergeThreshold = 20;
                }
                this.DockConnections();
            }
            return base.Connect(sourcePort, targetPort);
        }

        /// <summary>
        /// Tests to see if a connection between the two given ports is allowed.
        /// </summary>
        /// <param name="sourcePort">Source port to test.</param>
        /// <param name="targetPort">Target port to test.</param>
        /// <returns>
        /// True if a connection is allowed between the two ports; otherwise False.
        /// </returns>
        /// <remarks>
        /// <para>
        /// This method gives the symbol an opportunity to reject a connection
        /// between two ports. Both port containers being connected are asked
        /// for permission to create the connection.
        /// </para>
        /// <para>
        /// The default implementation always returns True.
        /// </para>
        /// <para>
        /// This code checks to see if the connection that is being created or which is being moved
        ///does not get to either of these..
        /// 1)corrected to its other parent 
        /// 2)To it's other parent's other connections (As we are not allwoing duplication of
        /// connections between nodes)
        /// And allow the connection to move when it is being shifted to some other port of
        /// its container node Symbol
        /// </para>
        /// <seealso cref="Syncfusion.Windows.Forms.Diagram.IPortContainer"/>
        /// <seealso cref="Syncfusion.Windows.Forms.Diagram.Port"/>
        /// </remarks>
        public override bool AcceptConnection(Port sourcePort, Port targetPort)
        {
            //When there are multiple links ending at a port, sometimes when a link port is clicked, 
            //Connect Event is fired to connect one link port with another, and thus results in disconnecting
            //those link ports from there node. Thus we are checking here if the AcceptConnection is
            //asked to accept a connection between two linkPorts then we are rejecting that connection.
            if ((sourcePort is LinkPort) && (targetPort is LinkPort))
            {
                return false;
            }


            bool validconnection = true;
            if ((null != this.ToNode) && (null != this.FromNode))
            {
                if (sourcePort == this.TailPort)
                {
                    if (targetPort.Parent == this.ToNode)
                    {
                        validconnection = false;
                    }
                    else if (targetPort.Parent == this.FromNode)
                    {
                        validconnection = true;
                    }
                    else
                    {
                        if (0 < ((this.ToNode as Ari_FlowNodeSymbol).Edges).Count)
                        {
                            foreach (Link link in (this.ToNode as Ari_FlowNodeSymbol).Edges)
                            {
                                if (((link.ToNode as Ari_FlowNodeSymbol) == (targetPort.Parent as Ari_FlowNodeSymbol)) || ((link.FromNode as Ari_FlowNodeSymbol) == (targetPort.Parent as Ari_FlowNodeSymbol)))
                                {
                                    validconnection = false;
                                }
                            }
                        }
                    }
                }
                else if (sourcePort == this.HeadPort)
                {
                    if (targetPort.Parent == this.FromNode)
                    {
                        validconnection = false;
                    }
                    else if (targetPort.Parent == this.ToNode)
                    {
                        validconnection = true;
                    }
                    else
                    {
                        if (0 < ((this.FromNode as Ari_FlowNodeSymbol).Edges).Count)
                        {
                            foreach (Link link in (this.FromNode as Ari_FlowNodeSymbol).Edges)
                            {
                                if (((link.FromNode as Ari_FlowNodeSymbol) == (targetPort.Parent as Ari_FlowNodeSymbol)) || ((link.ToNode as Ari_FlowNodeSymbol) == (targetPort.Parent as Ari_FlowNodeSymbol)))
                                {
                                    validconnection = false;
                                }
                            }
                        }
                    }
                }
            }
            if (!validconnection)
            {
                return validconnection;
            }
            else
            {
                return base.AcceptConnection(sourcePort, targetPort);
            }
        }
        /// <summary>
        /// Called after a change is made to the set of connections belonging to
        /// the link.
        /// </summary>
        /// <param name="evtArgs">Event arguments.</param>
        /// <remarks>
        /// <para>
        /// We have used this event to set the page Dirty when ever a connection is moved
        /// from one port to another.
        /// </para>
        /// <para>
        /// Docks the connection.
        /// </para>
        /// </remarks>
        protected override void OnConnectionsChangeComplete(ConnectionCollectionEventArgs evtArgs)
        {
            base.OnConnectionsChangeComplete(evtArgs);
            if (null != ConnectionChanged)
            {
                ConnectionChanged();
            }
        }

        # endregion
    }

    /// <summary>
    /// </summary>
    [Serializable()]
    public class MyLine : OrthogonalLine
    {
        # region Constructor
        /// <summary>
        /// Constructs an orthogonal line given the two end points and the CompassHeadings for the end points points.
        /// </summary>
        /// <param name="pt1">First point.</param>
        /// <param name="headingpt1">First CompassHeading.</param>
        /// <param name="pt2">Second point.</param>
        /// <param name="headingpt2">Second CompassHeading</param>
        public MyLine(PointF pt1, CompassHeading headingpt1, PointF pt2, CompassHeading headingpt2)
        {
            if ((headingpt1 == CompassHeading.None) || (headingpt2 == CompassHeading.None))
                Geometry.CalcEndpointDirections(pt1, pt2, out headingpt1, out headingpt1);

            float padLeft = this.PaddingLeft;
            float padRight = this.PaddingRight;
            float padTop = this.PaddingTop;
            float padBottom = this.PaddingBottom;

            PointF[] pts = Geometry.CalcOrthogonalPoints(pt1, headingpt1, pt2, headingpt2, padLeft, padRight, padTop, padBottom);

            if (pts != null)
            {
                this.grfxPath = this.CreateGraphicsPath(pts);
            }

            // Assign values directly to propertyValues hashtable to avoid
            // firing property change event, which would cause the points
            // to be recalculated.
            this.propertyValues["HeadingEndPoint1"] = headingpt1;
            this.propertyValues["HeadingEndPoint2"] = headingpt2;
        }

        // Serialization constructor
        protected MyLine(SerializationInfo info, StreamingContext context)
            : base(info, context)
        {
        }

        [SecurityPermission(SecurityAction.Demand, SerializationFormatter = false)]
        protected override void GetObjectData(SerializationInfo info, StreamingContext context)
        {
            info.SetType(this.GetType());
            base.GetObjectData(info, context);
        }

        // Creates a new object that is a copy of the current instance.	
        // An override of the Clone method is required for clipboard copy/paste.
        public override object Clone()
        {
            return new MyLine(this);
        }

        public MyLine(MyLine src)
            : base(src)
        {
            // code here...
        }
        # endregion
    }
}
